@@ -3,7 +3,7 @@ use lindera_core::LinderaResult;
 
 fn main() -> LinderaResult<()> {
     // create tokenizer
-    let mut tokenizer = Tokenizer::new()?;
+    let tokenizer = Tokenizer::new()?;
 
     // tokenize the text
     let tokens = tokenizer.tokenize("関西国際空港限定トートバッグ")?;
